@@ -1,8 +1,7 @@
 /*
  *      templates.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005 The Geany contributors
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -283,7 +282,7 @@ static void create_file_template_menu(void)
 
 	new_with_template_toolbar_menu = gtk_menu_new();
 	g_object_ref(new_with_template_toolbar_menu);
-	geany_menu_button_action_set_menu(GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")), 
+	geany_menu_button_action_set_menu(GEANY_MENU_BUTTON_ACTION(toolbar_get_action_by_name("New")),
 		new_with_template_toolbar_menu);
 }
 
@@ -615,8 +614,9 @@ static gchar *run_command(const gchar *command, const gchar *file_name,
 	}
 	else
 	{
-		g_warning(_("Cannot execute command \"%s\" from the template: %s. "
-			"Check the path in the template."), command, error->message);
+		g_warning(_("Cannot execute template command \"%s\". "
+			"Hint: incorrect paths in the command are a common cause of errors. "
+			"Error: %s."), command, error->message);
 		g_error_free(error);
 	}
 
